# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
from trytond.i18n import gettext
from trytond.model import fields
from trytond.pool import PoolMeta
from . import luhn
from .exceptions import SIRETValidationError


class Address(metaclass=PoolMeta):
    __name__ = 'party.address'

    siret_nic = fields.Char("SIRET NIC", select=True, size=5)
    siret = fields.Function(fields.Char('SIRET'), 'get_siret')

    def get_siret(self, name):
        if self.party.siren and self.siret_nic:
            return self.party.siren + self.siret_nic

    @classmethod
    def validate(cls, addresses):
        super(Address, cls).validate(addresses)
        for address in addresses:
            address.check_siret()

    def check_siret(self):
        '''
        Check validity of SIRET
        '''
        if self.siret:
            if (len(self.siret) != 14
                    or not luhn.validate(self.siret)):
                raise SIRETValidationError(
                    gettext('party_siret.msg_invalid_siret',
                        number=self.siret,
                        address=self.rec_name))



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

try:
    from trytond.modules.party_siret.tests.test_party_siret import suite
except ImportError:
    from .test_party_siret import suite

__all__ = ['suite']



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.


def validate(value):
    """
    Validate value with Luhn algorithm
    :param value: the value
    :return: a boolean
    """
    if not isinstance(value, str):
        value = str(value)
    try:
        evens = sum(int(x) for x in value[-1::-2])
        odds = sum(sum(divmod(int(x) * 2, 10)) for x in value[-2::-2])
        return (evens + odds) % 10 == 0
    except ValueError:
        return False



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from trytond.pool import Pool
from . import party
from . import address


def register():
    Pool.register(
        party.Party,
        address.Address,
        module='party_siret', type_='model')



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
from trytond.i18n import gettext
from trytond.model import fields
from trytond.pool import PoolMeta
from . import luhn
from .exceptions import SIRENValidationError


class Party(metaclass=PoolMeta):
    __name__ = 'party.party'

    siren = fields.Char("SIREN", select=True, size=9)

    @classmethod
    def validate(cls, parties):
        super(Party, cls).validate(parties)
        for party in parties:
            party.check_siren()

    def check_siren(self):
        '''
        Check validity of SIREN
        '''
        if self.siren:
            if len(self.siren) != 9 or not luhn.validate(self.siren):
                raise SIRENValidationError(
                    gettext('party_siret.msg_invalid_siren',
                        number=self.siren,
                        party=self.rec_name))



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import ModuleTestCase
from trytond.modules.party_siret import luhn


class PartySiretTestCase(ModuleTestCase):
    'Test PartySiret module'
    module = 'party_siret'

    def test_luhn(self):
        'Test luhn'
        values = (
            (4111111111111111, True),
            ('4111111111111111', True),
            (4222222222222, True),
            (378734493671000, True),
            (5424000000000015, True),
            (5555555555554444, True),
            (1008, True),
            ('0000001008', True),
            ('000000001008', True),
            (4012888888881881, True),
            (1234567890123456789012345678909, True),
            (4111111111211111, False),
            (42222222222224, False),
            (100, False),
            ('100', False),
            ('0000100', False),
            ('abc', False),
            (None, False),
            (object(), False),
            )
        for value, test in values:
            self.assertTrue(luhn.validate(value) == test)


def suite():
    suite = trytond.tests.test_tryton.suite()
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
        PartySiretTestCase))
    return suite
